/********************************************************************
* 		copyright (C) 2014 all rights reserved
*			 @file: gprstest.c
* 		  @Created: 2014-12-16 16:00
* 	  	   @Author: Conway
* 	  @Description: test the GPRS Modules 
*	  @Modify Date: 2014-12-16 16:00
*********************************************************************/
#include <termios.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>

struct message_info {
	char cnnu[16];
	char phnu[16];
	char message[128];
};

struct pdu_info {
	char cnswap[32];
	char phswap[32];
};

/**
 * @brief: initialize the serial port 
 * @Param: port_fd: file descriptor of serial port
 */
void serial_init(int port_fd)
{
	struct termios options;
	tcgetattr(port_fd, &options);
	options.c_cflag |= ( CLOCAL | CREAD );
	options.c_cflag &= ~CSIZE;
	options.c_cflag &= ~CRTSCTS;
	options.c_cflag |= CS8;
	options.c_cflag &= ~CSTOPB; 
	options.c_iflag |= IGNPAR;
	options.c_oflag = 0;
	options.c_lflag = 0;

	cfsetispeed(&options, B115200);
	cfsetospeed(&options, B115200);
	tcsetattr(port_fd, TCSANOW, &options);
}

/**
 * @brief: swap two arrays 
 * @Param: number: phone number, source array 
 * @Param: swap: array converted
 */
void swap(char number[], char swap[])
{
	char ch1[] = "86";
	char tmp[16];
	int i;

	memset(swap, 0, 32);
	memset(tmp, 0, 16);
	strcpy(swap, number);
	strcat(swap, "f");
	strcat(ch1, swap);
	strcpy(swap, ch1);

	for(i = 0; i <= strlen(swap) - 1; i += 2){
		tmp[i + 1] = swap[i];
 		tmp[i] = swap[i + 1];
	}
	
	strcpy(swap, tmp);
}

/**
 * @brief: send message
 * @Param: port_fd: file descriptor of serial port
 * @Param: cmgf: SMS mode. 0:PDU mode  1:Text mode 
 * @Param: cmgs: phone number
 * @Param: message: SMS content
 */
int send(int port_fd, char *cmgf, char *cmgs, char *message)
{
	int nread, nwrite;
	char buff[300];
	char reply[300];

	memset(buff, 0, sizeof(buff));
	strcpy(buff, "at\r");
	nwrite = write(port_fd, buff, strlen(buff));
	printf("nwrite=%d, %s\n", nwrite, buff);

	memset(reply, 0, sizeof(reply));
	usleep(500000);
	nread = read(port_fd, reply, sizeof(reply));
	printf("nread=%d, %s\n", nread, reply);

	memset(buff, 0, sizeof(buff));
	strcpy(buff, "AT+CMGF=");
	strcat(buff, cmgf);
	strcat(buff,"\r");
	nwrite = write(port_fd, buff, strlen(buff));
	printf("nwrite=%d, %s\n", nwrite, buff);

	memset(reply, 0, sizeof(reply));
	usleep(500000);
	nread = read(port_fd, reply, sizeof(reply));
	printf("nread=%d, %s\n", nread, reply);

	memset(buff, 0, sizeof(buff));
	strcpy(buff, "AT+CMGS=");
	strcat(buff, cmgs);
	strcat(buff, "\r");
	nwrite = write(port_fd, buff, strlen(buff));
	printf("nwrite=%d, %s\n", nwrite, buff);

	memset(reply, 0, sizeof(reply));
	usleep(500000);
	nread = read(port_fd, reply, sizeof(reply));
	printf("nread=%d,%s\n", nread, reply);

	memset(buff, 0, sizeof(buff));
	strcpy(buff, message);
	nwrite = write(port_fd, buff, strlen(buff));
	printf("nwrite=%d,%s\n", nwrite, buff);

	memset(reply, 0, sizeof(reply));
	usleep(500000);
	nread = read(port_fd, reply, sizeof(reply));
	printf("nread=%d,%s\n", nread, reply);
}

/**
 * @brief: send an english message
 * @Param: port_fd: file descriptor of serial port
 * @Param: info: variable of message_info structure
 */
int send_en_message(int port_fd, struct message_info info)
{	
	char cmgf[] = "1";
	int count = 0;
 	char cmgs[16] = {'\0'};

	getchar();
	printf("enter recever phnumber :\n");
	gets(info.phnu);
	printf("enter you message !\n");
	gets(info.message);
	strcat(info.message,"\x1a");
	strcat(cmgs,info.phnu);

	send(port_fd, cmgf, cmgs, info.message);
}

/**
 * @brief: send a chinese message
 * @Param: port_fd: file descriptor of serial port
 * @Param: info: variable of message_info structure
 */
int send_cn_message(int port_fd, struct message_info info)
{
	char cmgf[] = "0";
	char cmgs[4] = {'\0'};
	char ch2[] = "0891";
	char ch3[] = "1100";
	char ch4[] = "000800";
	char ch5[] = "0d91";
	char final[300];
	char *message[3] = {
		"206DF157335E027C735C1479D1628000470050005200536A2157576D4B8BD5FF01",
		"56004D0059002D004700500052005300300030003700436A215757662F6DF157335E027C735C1479D1628067099650516C53F8670065B063A851FA76844E006B3E00470053004D002F00470050005200536A2157573002",
		"0e795D60A85DE54F5C61095FEBFF01"
	};
	struct pdu_info pdu;
	int count = 0, flag, len;
	getchar();
	memset(final, 0, 80);

	printf("enter your centre phnumber :\n");
	gets(info.cnnu);
	
	while(strlen(info.cnnu) != 11) {
		if(count >= 3) {
			printf("count out !\n");
			return -1;
		}
		printf("number shuld be --11-- bits ! enter agin :\n");
		gets(info.cnnu);
		count ++;
	}

	printf("enter your recever phnumber :\n");
	gets(info.phnu);
	
	while(strlen(info.phnu) != 11) {
		if(count >= 3) {
			printf("count out !\n");
			return -1;
		}
		printf("number shuld be --11-- bits ! enter agin :\n");
		gets(info.phnu);
		count++;
	}
	
	printf("choice message :\n");
	printf("1.message1.\n");
	printf("2.message2.\n");
	printf("3.message3.\n");
	scanf("%d", &flag);
	swap(info.phnu, pdu.phswap);
	swap(info.cnnu, pdu.cnswap);

	strcpy(final, ch2);
	strcat(final, pdu.cnswap);
	strcat(final, ch3);
	strcat(final, ch5);
	strcat(final, pdu.phswap);
	strcat(final, ch4);
	strcat(final, message[flag - 1]);
	strcat(final, "\x1a");

	len = strlen(ch3)+ strlen(ch4)+ strlen(ch5)+strlen(pdu.phswap)+ strlen(message[flag - 1]);
	puts(final);
	sprintf(cmgs, "%d", len/2);
	puts(final);
	send(port_fd, cmgf, cmgs, final);
}

/**
 * @brief: receive messages
 * @Param: port_fd: file descriptor of serial port
 * @Param: item: item of messages
 */
void receive_message(const int port_fd, char item[])
{
	int cnt, nread;
	char recv_buf[128];
	char buf[128];
	
	/* Flushes buffer */
	tcflush(port_fd, TCIOFLUSH);
	
	cnt = snprintf(buf, sizeof(buf), "AT+CMGF=1%c", 0x0D);
	if ((cnt = write(port_fd, buf, strlen(buf))) < 0) {
		perror("Write cmd error");
		return;
	}
	
	if (!strcmp(item, "ALL"))
		cnt = snprintf(buf, sizeof(buf), "AT+CMGL=%s%c", item, 0x0D);
	else
		cnt = snprintf(buf, sizeof(buf), "AT+CMGR=%s%c", item, 0x0D);
	if ((cnt = write(port_fd, buf, strlen(buf))) < 0) {
		perror("Write cmd error");
		return;
	}
	
	memset(recv_buf, 0, sizeof(recv_buf));
	if (!strcmp(item, "ALL"))
		sleep(7);
	else
		usleep(500000);
		
	do {
		nread = read(port_fd, recv_buf, 1);
		if (nread < 0) {
			break;
		}	
		printf("%s", recv_buf);
		memset(recv_buf, 0, sizeof(recv_buf));
	} while (nread == 1);
	
}

/**
 * @brief: make a telephone call
 * @Param: port_fd: file descriptor of serial port
 * @Param: phonenum: phone number
 */
void dial(const int port_fd,  const unsigned char *phonenum)
{
	int nread;
	char temp;
	unsigned char recv_buf[20];
	unsigned char send_data[20] = "ATD";

	strcat(send_data, phonenum);
	send_data[strlen(send_data)] = ';';
	send_data[strlen(send_data)] = 0x0d;

	if (write(port_fd, send_data, strlen(send_data)) < 0) {
		perror("Write error");
	} else {
		printf("calling ...\n");
	}
	
	do {
		nread = read(port_fd, recv_buf, 1);
		temp = recv_buf[0];
		if (recv_buf[0] == 'O')	
			printf("OK\n");
		memset(recv_buf, 0, sizeof(recv_buf));
	} while (temp != 'O');
}

/**
 * @brief: Hang up a phone
 * @Param: port_fd: file descriptor of serial port
 */
void hangup(const int port_fd)
{
	unsigned char send_data[5] = { 'A', 'T', 'H', 0x0d };

	if (write(port_fd, send_data, strlen(send_data)) < 0) {
		perror("Write error");
	}

	printf("Hang up\n");
}

/**
 * @brief: receive a phone
 * @Param: port_fd: file descriptor of serial port
 */
void receivephone(const int port_fd)
{
	unsigned char send_data[5] = { 'A', 'T', 'A', 0x0d };

	if (write(port_fd, send_data, strlen(send_data)) < 0) {
		perror("Write error");
	}

	printf("Receive phone\n");
}

/**
 * @brief: main function  
 * @Param: argc: number of parameters
 * @Param: argv: parameters list
 */
int main(int argc, char *argv[])
{
	int fd;
	char choice;
	char message_item[5];
	char phone_number[20];
	struct message_info info;
	
	if (argc != 2) {
		printf("Usage: %s [ serial port ] \n", argv[0]);
		return -1;	
	}
	
	fd = open(argv[1], O_RDWR|O_NOCTTY|O_NDELAY);
	
	if (-1 == fd) {
		perror("Can't open serial port");
		return -1;
	}
	
	serial_init(fd);
	
	while (1) {
		printf("\nMenu:");
		printf("\n================================\
				\n1.send english message.\
				\n2.send chinese message.\
				\n3.receive message.\
				\n4.dail.\
				\n5.receive phone.\
				\n6.hung up.\
				\n7.exit.\
				\n================================\
				\n>");
		choice = getchar();
		switch(choice)
		{
			case '1': 
				send_en_message(fd, info);
				break;
			case '2': 
				send_cn_message(fd, info);
				getchar();
				break;
			case '3': 
				getchar();
				printf("message item:");
				gets(message_item);
				receive_message(fd, message_item);
				break;
			case '4':
				getchar();
				printf("\nPhone num:");
				gets(phone_number);
				printf("\n");
				dial(fd, phone_number);
				break;
			case '5':
				receivephone(fd);
				break;
			case '6':
				getchar();
				hangup(fd);
				break;			
			case '7': 
				exit(0);
				break;
			default : 
				break;
		}
	}
	close(fd);
	return 0;
}
